@@ -9,7 +9,7 @@
  * Werner Koch <wk@gnupg.org>
  *
  * Copyright:
- * Copyright (C) 2009-2015 Greenbone Networks GmbH
+ * Copyright (C) 2009-2016 Greenbone Networks GmbH
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -147,6 +147,7 @@ typedef struct
   gboolean scan_end;            ///< TRUE if the time the scan finished should be included
   guint autofp;                 ///< Whether to trust vendor security updates. 0 No, 1 full match, 2 partial.
   gint timeout;                 ///< Timeout of request.
+  gboolean empty_as_unknown;    ///< TRUE if empty results should produce an UNKNOWN response instead of OK
 } cmd_status_opts_t;
 
 /* Prototypes.  */
@@ -373,11 +374,13 @@ filter_report (entity_t report, const char *host_filter,
   entity_t results;
   entities_t elems;
   entity_t elem;
+  entity_t errors;
   int any_found = 0;
   int high_count = 0;
   int medium_count = 0;
   int low_count = 0;
   int log_count = 0;
+  int error_count = 0;
   int response_code = NAGIOS_OK;
   GPtrArray *high_oids = NULL;
   GPtrArray *medium_oids = NULL;
@@ -649,6 +652,39 @@ filter_report (entity_t report, const char *host_filter,
       elems = next_entities (elems);
     }
 
+  errors = entity_child (report, "errors");
+  if (errors != NULL)
+    {
+      if (host_filter)
+        {
+          entities_t error_elements;
+          entity_t error_element;
+
+          error_elements = errors->entities;
+          while ((error_element = first_entity (error_elements)))
+            {
+              if (strcmp (entity_name (error_element), "error") == 0)
+                {
+                  entity_t error_host;
+
+                  error_host = entity_child (error_element, "host");
+                  if (strcmp (entity_text (error_host), host_filter) == 0)
+                    error_count++;
+                }
+              error_elements = next_entities (error_elements);
+            }
+        }
+      else
+        {
+          entity_t count_child;
+          const char *error_text;
+
+          count_child = entity_child (errors, "count");
+          error_text = entity_text (count_child);
+          error_count = atoi (error_text);
+        }
+    }
+
   if (high_count > 0)
     {
       response_code = NAGIOS_CRITICAL;
@@ -658,6 +694,17 @@ filter_report (entity_t report, const char *host_filter,
       response_code = NAGIOS_WARNING;
     }
 
+  if ((results->entities == NULL) || (!any_found && host_filter))
+    {
+      if (status_opts.empty_as_unknown)
+        response_code = NAGIOS_UNKNOWN;
+    }
+
+  if ((error_count > 0) && (response_code == NAGIOS_OK))
+    {
+      response_code = NAGIOS_UNKNOWN;
+    }
+
   respond (response_code, "%i vulnerabilities found - High: %i Medium: %i Low: %i\n",
            (high_count + medium_count + low_count), high_count, medium_count, low_count);
 
@@ -666,6 +713,18 @@ filter_report (entity_t report, const char *host_filter,
   else if (!any_found && host_filter)
     respond_data ("Report did not contain vulnerabilities for IP %s\n", host_filter);
 
+  if (error_count > 0)
+    {
+      if (host_filter)
+        {
+          respond_data ("Report did contain %i errors for IP %s\n", error_count, host_filter);
+        }
+      else
+        {
+          respond_data ("Report did contain %i errors\n", error_count);
+        }
+    }
+
   if (status_opts.report_link)
     respond_data ("https://%s/omp?cmd=get_report&report_id=%s\n",
                   (gchar *) status_opts.manager_host, entity_attribute (report,
@@ -975,6 +1034,8 @@ main (int argc, char **argv)
   static gboolean display_log_messages = FALSE;
   static gboolean display_scan_end = FALSE;
   static guint autofp = 0;
+  static gboolean empty_as_unknown = FALSE;
+  static gboolean use_asset_management = FALSE;
   /* The rest of the args. */
   static gchar **rest = NULL;
 
@@ -1034,9 +1095,13 @@ main (int argc, char **argv)
      "Include timestamp of scan end in output", NULL},
     {"autofp", 0, 0, G_OPTION_ARG_INT, &autofp,
      "Trust vendor security updates for automatic false positive filtering (0=No, 1=full match, 2=partial).", "<n>"},
+    {"empty-as-unknown", 'e', 0, G_OPTION_ARG_NONE, &empty_as_unknown,
+     "Respond with UNKNOWN on empty results", NULL},
+    {"use-asset-management", 'A', 0, G_OPTION_ARG_NONE, &use_asset_management,
+     "Request host status via Asset Management", NULL},
     {G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &rest,
      NULL, NULL},
-    {NULL}
+    {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
   if (setlocale (LC_ALL, "") == NULL)
@@ -1057,7 +1122,7 @@ main (int argc, char **argv)
   if (print_version)
     {
       printf ("Check-OMP Nagios Command Plugin %s\n", OPENVASCLI_VERSION);
-      printf ("Copyright (C) 2015 Greenbone Networks GmbH\n");
+      printf ("Copyright (C) 2016 Greenbone Networks GmbH\n");
       printf ("License GPLv2+: GNU GPL version 2 or later\n");
       printf
         ("This is free software: you are free to change and redistribute it.\n"
@@ -1166,7 +1231,180 @@ main (int argc, char **argv)
     {
       entity_t status;
 
-      if (task_string == NULL)
+      if (use_asset_management)
+        {
+          if (host_filter == NULL)
+            {
+              exit_status =
+                respond (NAGIOS_UNKNOWN, "Status request via Asset Management requires host filter\n");
+            }
+          else
+            {
+              entity_t asset_report;
+              entity_t host_detail;
+              entities_t host_details;
+              gchar *report_id = NULL;
+              entity_t full_report;
+              cmd_status_opts_t status_opts;
+
+              int res;
+              int high_count = 0;
+              int medium_count = 0;
+              int low_count = 0;
+
+              omp_get_report_opts_t asset_opts = omp_get_report_opts_defaults;
+              omp_get_report_opts_t report_opts = omp_get_report_opts_defaults;
+
+              asset_opts.overrides = overrides_flag;
+              asset_opts.autofp = autofp;
+              asset_opts.timeout = timeout;
+              asset_opts.type = "assets";
+              asset_opts.host = host_filter;
+
+              manager_open (connection);
+              res = omp_get_report_ext (&(connection->session), asset_opts, &asset_report);
+              if (res == 0)
+                {
+                  asset_report = entity_child (asset_report, "report");
+                  if (asset_report == NULL)
+                    {
+                      exit_status = respond (NAGIOS_UNKNOWN, "Failed to get first asset report wrapper\n");
+                    }
+                  else
+                    {
+                      asset_report = entity_child (asset_report, "report");
+                      if (asset_report == NULL)
+                        {
+                          exit_status =  respond (NAGIOS_UNKNOWN, "Failed to get first asset report\n");
+                        }
+                      else
+                        {
+                          asset_report = entity_child (asset_report, "host");
+                          if (asset_report == NULL)
+                            {
+                              exit_status =  respond (NAGIOS_UNKNOWN, "Failed to get asset host element\n");
+                            }
+                          else
+                            {
+                              host_details = asset_report->entities;
+                              while ((host_detail = first_entity (host_details)))
+                                {
+                                  if (strcmp (entity_name (host_detail), "detail") == 0)
+                                    {
+                                      entity_t name;
+                                      entity_t value;
+
+                                      name = entity_child (host_detail, "name");
+                                      value = entity_child (host_detail, "value");
+
+                                      if (strcmp (entity_text (name), "report/@id") == 0)
+                                        report_id = g_strdup (entity_text (value));
+                                      if (strcmp (entity_text (name), "report/result_count/high") == 0)
+                                        high_count = atoi (entity_text (value));
+                                      if (strcmp (entity_text (name), "report/result_count/medium") == 0)
+                                        medium_count = atoi (entity_text (value));
+                                      if (strcmp (entity_text (name), "report/result_count/low") == 0)
+                                        low_count = atoi (entity_text (value));
+                                    }
+                                  host_details = next_entities (host_details);
+                                }
+
+                              if (report_id == NULL)
+                                {
+                                  exit_status = respond (NAGIOS_UNKNOWN, "Failed to get report_id via Asset Management\n");
+                                }
+                              else
+                                {
+                                  if ((high_count + medium_count) == 0)
+                                    {
+                                      int response_code = NAGIOS_OK;
+                                      exit_status = respond (response_code,
+                                                             "%i vulnerabilities found - High: 0 Medium: 0 Low: %i\n",
+                                                             low_count, low_count);
+
+                                      if (report_link)
+                                        respond_data ("https://%s/omp?cmd=get_report&report_id=%s\n",
+                                                      (gchar *) (gpointer) connection->host_string, report_id);
+
+                                      if (display_scan_end)
+                                        respond_data ("SCAN_END: %s\n", entity_text (entity_child (asset_report, "end")));
+
+                                      respond_perf_data ("|High=%i Medium=%i Low=%i\n",
+                                                         high_count, medium_count, low_count);
+                                    }
+                                  else
+                                    {
+                                      report_opts.report_id = report_id;
+
+                                      status_opts.report_link = report_link;
+                                      status_opts.dfn_ids = display_dfn_ids;
+                                      status_opts.oids = display_oids;
+                                      status_opts.manager_host = (gpointer) connection->host_string;
+                                      status_opts.descr = display_descriptions;
+                                      status_opts.log_messages = display_log_messages;
+                                      status_opts.scan_end = display_scan_end;
+                                      status_opts.autofp = autofp;
+                                      status_opts.timeout = timeout;
+                                      status_opts.empty_as_unknown = empty_as_unknown;
+
+                                      report_opts.apply_overrides = overrides_flag;
+                                      report_opts.autofp = status_opts.autofp;
+                                      report_opts.timeout = status_opts.timeout;
+
+                                      if (!display_log_messages)
+                                        report_opts.levels = "hml";
+
+                                      res = omp_get_report_ext (&(connection->session), report_opts, &full_report);
+                                      if (res == 0)
+                                        {
+                                          full_report = entity_child (full_report, "report");
+                                          if (full_report == NULL)
+                                            {
+                                              exit_status = respond (NAGIOS_UNKNOWN,
+                                                                     "Failed to get first full report wrapper\n");
+                                            }
+                                          else
+                                            {
+                                              full_report = entity_child (full_report, "report");
+                                              if (full_report == NULL)
+                                                {
+                                                  exit_status = respond (NAGIOS_UNKNOWN,
+                                                                         "Failed to get first full report\n");
+                                                }
+                                              else
+                                                {
+
+                                                  exit_status = filter_report (full_report, host_filter, status_opts);
+                                                }
+                                            }
+                                        }
+                                      else if (res == 2)
+                                        {
+                                          exit_status = respond (NAGIOS_UNKNOWN,
+                                                                 "Timeout while getting full report.\n");
+                                        }
+                                      else
+                                        {
+                                          exit_status = respond (NAGIOS_UNKNOWN,
+                                                                 "Failed to get full report.\n");
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+              else if (res == 2)
+                {
+                  exit_status = respond (NAGIOS_UNKNOWN, "Timeout while getting asset report.\n");
+                }
+              else
+                {
+                  exit_status = respond (NAGIOS_UNKNOWN, "Failed to get asset report.\n");
+                }
+            }
+        }
+      else if (task_string == NULL)
         {
           exit_status =
             respond (NAGIOS_UNKNOWN, "Status request requires task name\n");
@@ -1198,6 +1436,7 @@ main (int argc, char **argv)
           status_opts.scan_end = display_scan_end;
           status_opts.autofp = autofp;
           status_opts.timeout = timeout;
+          status_opts.empty_as_unknown = empty_as_unknown;
 
           /* Returns 0 on success, 2 on timeout, -1 or OMP code on error.  */
           switch (omp_get_tasks_ext (&(connection->session), opts, &status))
