@@ -7,7 +7,7 @@
  * Michael Wiegand <michael.wiegand@greenbone.net>
  *
  * Copyright:
- * Copyright (C) 2009, 2010, 2015 Greenbone Networks GmbH
+ * Copyright (C) 2009, 2010, 2015, 2016 Greenbone Networks GmbH
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -1149,7 +1149,7 @@ main (int argc, char **argv)
     /* Enable details */
     {"details", 0, 0, G_OPTION_ARG_NONE, &cmd_details,
      "Enable detailed view.", NULL},
-    {NULL}
+    {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
   if (setlocale (LC_ALL, "") == NULL)
@@ -1180,7 +1180,7 @@ main (int argc, char **argv)
   if (print_version)
     {
       printf ("OMP Command Line Interface %s\n", OPENVASCLI_VERSION);
-      printf ("Copyright (C) 2010-2015 Greenbone Networks GmbH\n");
+      printf ("Copyright (C) 2010-2016 Greenbone Networks GmbH\n");
       printf ("License GPLv2+: GNU GPL version 2 or later\n");
       printf
         ("This is free software: you are free to change and redistribute it.\n"
